/*
  A build tool to intercept calls to "ar" and "CC" and do the right
  thing in the context of KDE and the GNU-isms it expects.

  In the case of "CC", run the CC prelinker prior to creating libfoo.so

  Also accept a global -R variable, impervious to whims of makefiles
  
  Also, KDE makefiles frequently use libraries via direct reference,
  i.e. "a/b/libfoo.so", rather than "-L a/b -l foo".  The
  linker can more or less handle this, but it always wants to hunt
  down any DT_NEEDED references in the library, even if one of those
  other needed shared objects is already specified elsewhere in the
  link line, possibly with a different name (libbar.so (link line)
  intead of libbar.so.1 (DT_NEEDED)).  Sticking in extra -L's for each
  of the directly referenced .so's seems to generate a good link.

  remove well meaning "-lc" arg
*/

#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <unistd.h>


char *ME = "xenv_CC";
int testing = 0;

/* pull in shared functions */
#include "xenv_funcs.c"

int changes = 0;


main(int argc, char **argv, char **envp)
{
  int i, i1, i2;
  char *nargv[5000];
  char *xtra_Ls[1000];
  int xtra_L_cnt = 0;
  char *p, *dirpath, *libname;
  int dasho;
  char *separator;
  int prelink_requested = 0;
  int linking = 1;		/* assume true, unless we see "-c" or "-E" */
  int n_obj = 0;
  int recursion_level;

  recursion_level = get_recursion_level();
  if ((recursion_level != 0) && (recursion_level != 1)) {
    die ("recursion level mismatch");
  }
  if (recursion_level == 1) {	/* no need to intervene */
    execve ("/usr/ccs/bin/CC", argv, envp);
    die ("exec failed????");
  }
  bump_recursion_level();

  if (getenv("XENVTEST") != NULL)
    testing = 1;

  /* loop through input args once to get the lay of the land */
  for (i = 1; i < argc; i++) {
    if (argv[i][0] == '-')
      switch (argv[i][1]) {
      case 'c':
      case 'E':
	linking = 0;
	break;

      case 'l':
	check_R (argv[i], argv[i+1]);
	break;

      default:
	if ((strcmp (argv[i], "-Tprelink_objects")) == 0)
	  prelink_requested = 1;
	break;
      }
    else {
      int l = strlen(argv[i]);
      if ((l > 2) && (strcmp (argv[i] + (l - 2), ".o") == 0) && (strcmp (argv[i-1], "-o") != 0)) {
	n_obj++;
      }
      if ((l > 3) && (strcmp (argv[i] + (l - 3), ".so") == 0) && (strcmp (argv[i-1], "-o") != 0)) {
	check_R (argv[i], argv[i+1]);
      }
    }
  }

  /* loop again, looking for pathname refs to shared objects */

  for (i = 1; i < argc; i++) {
    char *p2;

    p = strrchr (argv[i], '/');
    if (p == NULL)
      continue;
    libname = p + 1;
    separator = p;

    p2 = NULL;			/* find last ".so" sequence in name */
    p = libname;
    while (p = strstr (p, ".so")) {
      p2 = p;
      p++;
    }
    if (p2 == NULL)
      continue;			/* no ".so" in name */
    if ((*(p2+3) != '.') && (*(p2+3) != '\0')) /* not foo.so or foo.so.bar */
      continue;
    
    dasho = (strcmp (argv[i - 1], "-o") == 0);
    if (dasho)
      continue;			/* this lib is about to be created */

    dirpath = argv[i];
    *separator = '\0';

    p = mymalloc (strlen(dirpath) + 3);
    strcpy (p, "-L");
    strcat (p, dirpath);
    *separator = '/';
    changes++;
    xtra_Ls[xtra_L_cnt++] = p;
  }
  

  /* create a new argument list for CC */
  i1 = 0;			/* index to original input args */
  i2 = 0;			/* index to massaged arg list  */
  while (i1 < argc) {
    if (strcmp (argv[i1], "-lc") == 0) {
      i1++; changes++;
      continue;
    }
    if (strcmp (argv[i1], "-lutil") == 0) {
      die ("specified -lutil but that is most likely not intended");
    }
    if (strcmp (argv[i1], "-L/lib") == 0) {
      i1++; changes++;
      continue;
    }
    if (strcmp (argv[i1], "-L/usr/lib") == 0) {
      i1++; changes++;
      continue;
    }
    if (rpath_override && (argv[i1][0] == '-') && (argv[i1][1] == 'R')) {  /* remove -R passed in, add back later */
      i1 = i1 + ((argv[i1][2]) ? 1 : 2); /* "-Rfoo" or "-R foo" */
      changes++;
    }
    else {
normal:
      nargv[i2++] = argv[i1++];
    }
  }
  for (i = 0; i < xtra_L_cnt; i++) /* just stick them at the end, seems to work */
    nargv[i2++] = xtra_Ls[i];


  if (rpath_override && linking && new_R) {
    nargv[i2++] = "-R";
    nargv[i2++] = new_R;
  }

  nargv[i2++] = (char *) 0;


  if (n_obj && !prelink_requested) { /* at leat one foo.o, run prelinker unless it would happen twice */
    run_prelinker(argc, argv, envp);
  }

  if (changes || testing) {
    printf ("%s: %d changes :", ME, changes);
    for (i = 0; i < i2; i++)
      printf (" %s", nargv[i]);
    printf ("\n");
  }
  
  if (testing) {
    exit(0);
  }


  execve ("/usr/ccs/bin/CC", nargv, envp);
  die ("exec failed????");
  /* not reached */

}
