/* common routines for xenv_CC and xenv_ar */


void die (char *p)
{
  perror (p);
  write (2, ME, strlen(ME));
  write (2, " failed\n", 8);
  exit (1);
}

char *mymalloc (int n)
{
  char *p = malloc (n);
  if (p == NULL)
    die ("no memory???");
  return p;
}

void do_forkexec (char * cmd, char **argv, char **envp)
{
  int i, ec;
  pid_t childpid;
  int exit_stat;
  int low, high;

  if (testing)
    printf ("do_forkexec...  ");

  printf ("%s: [%s] %s", ME, cmd, argv[0]);
  for (i = 1; argv[i]; i++)
    printf (" %s", argv[i]);
  printf ("\n");

  if (testing)
    return;

  childpid = fork();
  if (childpid == -1)
    die ("fork fail");

  if (childpid != 0) {
    ec = wait (&exit_stat);
    if (ec == -1)
      die ("wait");
    if (exit_stat == 0)
      return;

    low = exit_stat & 0xff;
    high = (exit_stat >> 8) && 0xff;

    if (low == 0) {
      fprintf (stderr, "%s: failure of %s, exit = %x\n", ME, cmd, high);
      exit (high);		/* pass along exit status */
    }
    fprintf (stderr, "unepected failure of %s, wait status = %x\n", cmd, exit_stat);
    exit (1);
  }
  else {
    execve (cmd, argv, envp);
    die ("exec failed");
  }
}
  
/* remove a foo.ii template related file if one exists for foo.o */

void delete_ii (char *obj_file)
{
  char *ii_file, *p;
  int st;

  ii_file = mymalloc (strlen (obj_file) + 2);
  strcpy (ii_file, obj_file);
  p = ii_file + strlen (ii_file) - 1;
  *p++ = 'i';
  *p++ = 'i';
  *p++ = '\0';
  if (testing) {
    printf ("would try: unlink <%s>\n", ii_file);
    return;
  }
  st = unlink (ii_file);
  if (st == 0)
    printf ("%s: deleted %s\n", ME, ii_file);
  free (ii_file);
}

/* gather up list of ".o" files and run prelinker against them */

void run_prelinker (int argc, char **argv, char **envp)
{
  int i, l;
  int j = 0;
  char **cmd_args;
  int n_obj = 0;

  cmd_args = (char **) mymalloc ((argc + 4) * sizeof (char*));
  cmd_args[j++] = "CC";
  cmd_args[j++] = "-Tprelink_objects";
  cmd_args[j++] = "-Wt,-r";
  for (i = 1; i < argc; i++) {
    l = strlen(argv[i]);
    if ((l > 2) && (strcmp (argv[i] + (l - 2), ".o") == 0) && (strcmp (argv[i-1], "-o") != 0)) {
      cmd_args[j++] = argv[i];
      delete_ii (argv[i]);	/* thump instantiations from a possible previous linker pass */
      n_obj++;
    }
  }
  cmd_args[j++] = NULL;

  if (n_obj == 0)
    die ("trying to prelink but no .o files");

  do_forkexec ("/usr/ccs/bin/CC", cmd_args, envp);
}

int get_recursion_level ()
{
  char *p = getenv("XENVLEVEL");
  if (p == NULL)
    die ("XENVLEVEL not set");
  return (*p - '0');
}

void bump_recursion_level ()
{
  char *p = getenv("XENVLEVEL");
  if (p == NULL)
    die ("XENVLEVEL not set");
  *p = *p + 1;
}


/* libs can be specified as "-l foo", "-lfoo", and "x/y/foo.so".  Any
   time we find a lib that does not live in /usr/lib, we asssume that
   the BUILD_RPATH must be applied to the link target */

char *new_R = NULL;
int rpath_override = 0;

void check_R (char *arga, char *argb)
{
  static char *xenv_R_stringp = NULL;
  static int first_call = 1;
  char *libname, *p;
  char buf[1024];
  int l;

  if (first_call) {
    first_call = 0;

    xenv_R_stringp = getenv ("BUILD_RPATH");
    if (!xenv_R_stringp)
      return;
    
    rpath_override = 1;
  }

  if (!rpath_override)
    return;

  if (*xenv_R_stringp == '\0')
    return;  /* empty string means override with no RPATH */

  if (new_R)
    return;     /* already needed */


  l = strlen (arga);
  if ((l > 3) && (strcmp (arga + (l - 3), ".so") == 0)) {
    if (strncmp (arga, "/usr/lib/", 9)) {
      new_R = xenv_R_stringp;
      return;
    }
  }

  if (strlen (arga) == 2)
    libname = argb;
  else
    libname = arga + 2;

  if (!libname || (libname[0] == '\0'))
    die ("empty -l argument");

  if (strlen (libname) > (sizeof (buf) - 30))
    die ("-l argument too big");

  strcpy (buf, "/usr/lib/lib");
  strcat (buf, libname);
  strcat (buf, ".so");
  if (access (buf, F_OK) != 0)
    new_R = xenv_R_stringp;

}
